@@ -28,13 +28,16 @@
 #elif defined(__NetBSD__)
     #include <sys/types.h>
     #include <sys/sysctl.h>
+#elif defined(__HAIKU__)
+    #include <OS.h>
+    #include <image.h>
 #endif
 
 enum { FF_PIPE_BUFSIZ = 8192 };
 
-static inline int ffPipe2(int *fds, int flags)
+static inline int ffPipe2(int* fds, int flags)
 {
-    #ifdef __APPLE__
+    #ifndef FF_HAVE_PIPE2
         if(pipe(fds) == -1)
             return -1;
         fcntl(fds[0], F_SETFL, fcntl(fds[0], F_GETFL) | flags);
@@ -276,8 +279,10 @@ void ffProcessGetInfoLinux(pid_t pid, FFstrbuf* processName, FFstrbuf* exe, cons
     psinfo_t proc;
     if (ffReadFileData(filePath, sizeof(proc), &proc) == sizeof(proc))
     {
-        ffStrbufSetS(exe, proc.pr_psargs);
-        ffStrbufSubstrBeforeFirstC(exe, ' ');
+        const char* args = proc.pr_psargs;
+        if (args[0] == '-') ++args;
+        const char* end = strchr(args, ' ');
+        ffStrbufSetNS(exe, end ? (uint32_t) (end - args) : (uint32_t) strlen(args), args);
     }
 
     if (exePath)
@@ -301,10 +306,30 @@ void ffProcessGetInfoLinux(pid_t pid, FFstrbuf* processName, FFstrbuf* exe, cons
     if (proc)
     {
         char** argv = kvm_getargv(kd, proc, 0);
-        if (argv) ffStrbufSetS(exe, argv[0]);
+        if (argv)
+        {
+            const char* arg0 = argv[0];
+            if (arg0[0] == '-') arg0++;
+            ffStrbufSetS(exe, arg0);
+        }
     }
     kvm_close(kd);
 
+    #elif defined(__HAIKU__)
+
+    image_info info;
+    int32 cookie = 0;
+
+    while (get_next_image_info(pid, &cookie, &info) == B_OK)
+    {
+        if (info.type != B_APP_IMAGE) continue;
+        ffStrbufSetS(exe, info.name);
+
+        if (exePath)
+            ffStrbufSet(exePath, exe);
+        break;
+    }
+
     #endif
 
     if(exe->length == 0)
@@ -471,6 +496,18 @@ const char* ffProcessGetBasicInfoLinux(pid_t pid, FFstrbuf* name, pid_t* ppid, i
     if (!proc)
         return "kvm_getprocs() failed";
 
+    #elif defined(__HAIKU__)
+
+    team_info info;
+    if (get_team_info(pid, &info) == B_OK)
+    {
+        ffStrbufSetS(name, info.name);
+        if (ppid)
+            *ppid = info.parent;
+    }
+
+    FF_UNUSED(tty);
+
     #else
 
     return "Unsupported platform";
